﻿using System;
using System.Linq;
using System.Net.WebSockets;
using System.Threading.Tasks;
using System.Threading;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Web;
using System.Text.RegularExpressions;

/// <summary>
/// Игрок
/// </summary>
public class Player
{

    /// <summary>
    /// Веб-сокет игрока
    /// </summary>
    public WebSocket Socket { get; set; }

    /// <summary>
    /// ID комнаты
    /// </summary>
    public string RoomID { get; set; }

    /// <summary>
    /// ID игрока. Равен ключу в словаре.
    /// </summary>
    public readonly string ID;

    /// <summary>
    /// Nickname игрока
    /// </summary>
    public string Nickname { get; set; }

    /// <summary>
    /// Сколько подряд пропущено ходов
    /// </summary>
    public int TurnPassInSequence = 0;

    /// <summary>
    /// Сколько пропущено ходов всего
    /// </summary>
    public int TurnPassAll = 0;


    /// <summary>
    /// Отправка данных игроку
    /// </summary>
    /// <param name="data">Данные</param>
    public async Task SendData(Сommunication.Responce data)
    {
        try
        {
            if (Socket.State == WebSocketState.Open)
            {
                string ans = JsonConvert.SerializeObject(data);
                var answerBytes = new ArraySegment<byte>(Encoding.UTF8.GetBytes(ans));
                await Socket.SendAsync(answerBytes, WebSocketMessageType.Text, true, CancellationToken.None);
            }
        }
        catch (ObjectDisposedException)
        {
            // Что-то пошло не так.
            // Лайфхак: тогда ничего не делаем.
        }
    }

    /// <summary>
    /// Обрабатывает запрос от игрока
    /// </summary>
    /// <param name="req">Запрос</param>
    public async Task PlayerRequestHandler(Сommunication.Request req)
    {
        if (req["method"] == null)
        {
            var _data = new JObject();
            _data["text"] = "Method isn't set.";
            SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
            return;
        }
        if (req["method"].ToString() == "getManualID")
        {
            var _data = new JObject();
            _data["playerID"] = ID;
            SendData(new Сommunication.Responce(Сommunication.MessageType.Command, "PlayerStatus", _data));
            return;
        }
        if (req["method"].ToString() == "roomFind")
        {
            // Установка ника для игрока.
            if (req["nickname"] == null)
            {
                var _data = new JObject();
                _data["text"] = "Nickname isn't set.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                return;
            }
            else
            {
                if (req["nickname"].ToString().Length > 15 || req["nickname"].ToString().Length == 0 || !(new Regex(@"^[a-zA-Zа-яА-Я0-9: \.'\-_!@\^\?]{1,}$").IsMatch(req["nickname"].ToString())))
                {
                    var _data = new JObject();
                    _data["text"] = "Nickname's length/chars incorrect.";
                    SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                    return;
                }
                else
                {
                    Nickname = HttpUtility.HtmlEncode(req["nickname"].ToString());
                }
            }
            // Непосредственно приступаем к поиску комнаты.
            RoomFind();
            return;
        }
        if (req["method"].ToString() == "testFlags")
        {
            if (RoomID == null)
            {
                var _data = new JObject();
                _data["text"] = "You're not in the room.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
            }
            else
            {
                var a = req["flags"].ToObject<int[][]>();
                Global.Rooms[RoomID].TestFlags(this, a);
            }
            return;
        }


        if (req["method"].ToString() == "sendHint")
        {
            if (RoomID == null)
            {
                var _data = new JObject();
                _data["text"] = "You're not in the room.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
            }
            else
            {
                if (ID != Global.Rooms[RoomID].TurnBy)
                {
                    int _x, _y;
                    if (int.TryParse(req["x"].ToString(), out _x) && int.TryParse(req["y"].ToString(), out _y))
                    {
                        Global.Rooms[RoomID].SendHint(_x, _y);
                    }
                    else
                    {
                        var _data = new JObject();
                        _data["text"] = "Cell position incorrect.";
                        SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                    }
                }
                else
                {
                    var _data = new JObject();
                    _data["text"] = "Turn is your.";
                    SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                }
            }
            return;
        }


        if (req["method"].ToString() == "toChat")
        {
            if (req["msg"].ToString() == null)
            {
                var _data = new JObject();
                _data["text"] = "Message is empty.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
            }
            else
            {
                string m = req["msg"].ToString();
                m = new Regex("[ ]{1,}").Replace(m, " ");
                m = new Regex("[\r\n]{1,}").Replace(m, Environment.NewLine);
                if (new Regex("^[\r\n ]{0,}$").IsMatch(m))
                {
                    var _data = new JObject();
                    _data["text"] = "Message is hack-empty.";
                    SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                }
                else
                {
                    if (m.Length > 255)
                    {
                        var _data = new JObject();
                        _data["text"] = "Message length too long.";
                        SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                    }
                    else
                    {
                        Global.Rooms[RoomID].ToChat(this, HttpUtility.HtmlEncode(m));
                    }
                }
            }
            return;
        }





        if (req["method"].ToString() == "openCell")
        {
            if (RoomID == null)
            {
                var _data = new JObject();
                _data["text"] = "You're not in the room.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
            }
            else
            {
                if (req["cells"] != null)
                {
                    try
                    {
                        var c = req["cells"].ToObject<int[][]>();
                        Global.Rooms[RoomID].OpenCell(this, c, req["byX"].ToString(), req["byY"].ToString());
                    }
                    catch
                    {
                        var _data = new JObject();
                        _data["text"] = "Cell array parsing error.";
                        SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                    }
                }
                else
                {
                    var _data = new JObject();
                    _data["text"] = "Empty cell array.";
                    SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                }
            }
            return;
        }
        if (req["method"].ToString() == "getEasterEgg")
        {
            var _data = new JObject();
            _data["easterEgg"] = "Osetr — pidor!";
            SendData(new Сommunication.Responce(Сommunication.MessageType.Warning, "EasterEgg", _data));

            return;
        }
        if (req["method"].ToString() == "debug")
        {
            var _data = new JObject();
            _data["playersCount"] = Global.Players.Count - 1;
            _data["roomsCount"] = Global.Rooms.Count;
            _data["waitersCount"] = Global.WaitingPlayers.Count;
            SendData(new Сommunication.Responce(Сommunication.MessageType.Warning, "DebugInfo", _data));
            return;
        }

        if (req["method"].ToString() == "createLobby")
        {
            // Установка ника для игрока.
            if (req["nickname"] == null)
            {
                var _data = new JObject();
                _data["text"] = "Nickname isn't set.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                return;
            }
            else
            {
                if (req["nickname"].ToString().Length > 15 || req["nickname"].ToString().Length == 0 || !(new Regex(@"^[a-zA-Zа-яА-Я0-9: \.'\-_!@\^\?]{1,}$").IsMatch(req["nickname"].ToString())))
                {
                    var _data = new JObject();
                    _data["text"] = "Nickname's length/chars incorrect.";
                    SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                    return;
                }
                else
                {
                    Nickname = HttpUtility.HtmlEncode(req["nickname"].ToString());
                }
            }
            // Непосредственно приступаем к созданию лобби.
            FriendLobbyCreate();
            return;
        }

        if (req["method"].ToString() == "joinLobby")
        {
            // Установка ника для игрока.
            if (req["nickname"] == null)
            {
                var _data = new JObject();
                _data["text"] = "Nickname isn't set.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                return;
            }
            if (req["nickname"].ToString().Length > 15 || req["nickname"].ToString().Length == 0 || !(new Regex(@"^[a-zA-Zа-яА-Я0-9: \.'\-_!@\^\?]{1,}$").IsMatch(req["nickname"].ToString())))
            {
                var _data = new JObject();
                _data["text"] = "Nickname's length/chars incorrect.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                return;
            }
            if(req["lobbyID"] == null)
            {
                var _data = new JObject();
                _data["text"] = "LobbyID isn't set.";
                SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, _data));
                return;
            }

            Nickname = HttpUtility.HtmlEncode(req["nickname"].ToString());
            // Непосредственно приступаем к присоединению к лобби.
            FriendLobbyJoin(req["lobbyID"].ToString());
            return;
        }



        var data = new JObject();
        data["text"] = "Method unknown.";
        SendData(new Сommunication.Responce(Сommunication.MessageType.Error, string.Empty, data));
    }

    /// <summary>
    /// Постоянно принимает данные от игрока
    /// </summary>
    public async Task ReceiveData()
    {
        try
        {
            while (true)
            {
                if (Socket.State == WebSocketState.Open || Socket.State == WebSocketState.Connecting)
                {
                    // Сформируем буфер для запросов пользователя.
                    var buffer = new ArraySegment<byte>(new byte[4096]);

                    // Ожидаем данные от пользователя в виде массива байт...
                    var reqBytes = await Socket.ReceiveAsync(buffer, CancellationToken.None);

                    // Переведем сырые данные в Request клиента.
                    var req = new Сommunication.Request(Encoding.UTF8.GetString(buffer.Array));

                    // Передаем полученные данные в обработчик
                    PlayerRequestHandler(req);
                }
                else
                {
                    break;
                }
            }
        }
        catch { }
    }



    /// <summary>
    /// Очищает ресурсы игрока
    /// </summary>
    /// <param name="p">Ссылка на игрока</param>
    /// <returns></returns>
    public static async Task DispoceFrom(string p)
    {

        if (Global.Players.ContainsKey(p))
        {
            string roomID = Global.Players[p].RoomID;
            Player toDelete;
            Global.Players.TryRemove(p, out toDelete);
            if (Global.Rooms.ContainsKey(roomID))
                Global.Rooms[roomID].SecondPlayerWasLeave();
        }
        Global.WaitingPlayers.TryRemove(p, false);
    }




    /// <summary>
    /// Комната была удалена.
    /// </summary>
    public async Task RoomDeletedEvent()
    {
        await Task.Run(() =>
        {
            RoomID = null;
            var data = new JObject();
            data["text"] = 400; // The room has been removed.
            SendData(new Сommunication.Responce(
                Сommunication.MessageType.Status, "RoomStatusUpdate", data));
        });
    }



    /// <summary>
    /// Подсоединяется к дружескому лобби
    /// </summary>
    /// <returns></returns>
    public async Task FriendLobbyJoin(string lobbyId)
    {
        await Task.Run(() =>
        {
            if (Global.WaitingPlayers.TryGetFriendlyLobby(lobbyId))
            {
                // такое лобби существует, подсоединяемся
                string roomID = Tools.GenerateID(); // ID для комнаты (рандомный).
                Global.Rooms.TryAdd(roomID, new Room(ID, lobbyId, roomID));
                // Комната создана. Теперь она самостоятельно разошлет игрокам дальнейшие инструкции.
            }
            else
            {
                var data = new JObject();
                data["text"] = "The lobby does not exist!";
                SendData(new Сommunication.Responce(
                    Сommunication.MessageType.Error, "LobbyStatus", data));
            }
        });
    }



    /// <summary>
    /// Создает дружеское лобби
    /// </summary>
    public async Task FriendLobbyCreate()
    {
        await Task.Run(() =>
        {
            if (RoomID == null) 
            {
                try
                {
                    Global.WaitingPlayers.TryAdd(ID, true);
                    // Отправим ответ "ждите"...
                    var data = new JObject();
                    data["text"] = "Waiting for another player...";
                    SendData(new Сommunication.Responce(
                        Сommunication.MessageType.Status, "LobbyCreateStatus", data));
                }
                catch (Exception e)
                {
                    // TODO: Что-то пошло не так...
                    var aa = new JObject();
                    aa["messageAA"] = e.Message;
                    aa["messageAA2"] = e.Source;
                    aa["messageAA3"] = e.StackTrace;
                    aa["messageAA4"] = e.TargetSite.Name;
                    var bb = new Сommunication.Responce(Сommunication.MessageType.ServerError, "Expection", aa);
                    SendData(bb);
                }
            }
            else
            {
                var data = new JObject();
                data["text"] = "100"; // You are already in the room.
                data["roomID"] = RoomID;
                SendData(new Сommunication.Responce(
                    Сommunication.MessageType.Error, string.Empty, data));
            }
        });
    }


    /// <summary>
    /// Поиск комнаты для игрока
    /// </summary>
    public async Task RoomFind()
    {
        await Task.Run(() =>
        {
            if (RoomID == null)
            {
                // Пытаемся получить доступ к записи в глобальные объекты,
                // Дабы предовратить запись новых ожидающих игроков.

                try
                {
                    // Попытаемся добавить себя в очередь.
                    Global.WaitingPlayers.TryAdd(ID, false);
                    string opponentID = Global.WaitingPlayers.TryGetPlayer(ID);
                    // Если получили ID игрока из очереди корректно
                    if (opponentID != null)
                    {
                        if (Global.Players.ContainsKey(opponentID))
                        {
                            try
                            {
                                string roomID = Tools.GenerateID(); // ID для комнаты (рандомный).
                                Global.Rooms.TryAdd(roomID, new Room(ID, opponentID, roomID));
                                // Комната создана. Теперь она самостоятельно разошлет игрокам дальнейшие инструкции.
                            }
                            catch
                            {
                                DispoceFrom(opponentID);
                            }

                        }
                        else
                        {
                            DispoceFrom(opponentID);
                        }
                    }
                    else
                    {
                        // Так как других игроков нет - просто отправим ответ "ждите".
                        var data = new JObject();
                        data["text"] = "Waiting for another player...";
                        SendData(new Сommunication.Responce(
                            Сommunication.MessageType.Status, "RoomSearchStatus", data));
                    }
                }
                catch (Exception e)
                {
                    // TODO: Что-то пошло не так...
                    var aa = new JObject();
                    aa["messageAA"] = e.Message;
                    aa["messageAA2"] = e.Source;
                    aa["messageAA3"] = e.StackTrace;
                    aa["messageAA4"] = e.TargetSite.Name;
                    var bb = new Сommunication.Responce(Сommunication.MessageType.ServerError, "Expection", aa);
                    SendData(bb);
                }

            }
            else
            {
                var data = new JObject();
                data["text"] = "100"; // You are already in the room.
                data["roomID"] = RoomID;
                SendData(new Сommunication.Responce(
                    Сommunication.MessageType.Error, string.Empty, data));
            }
        });
    }

    /// <summary>
    /// Основной конструктор для игрока.
    /// Начинает автоматическое прослушивание клиента.
    /// </summary>
    /// <param name="socket">Веб-сокет игрока</param>
    /// <param name="playerID">ID игрока (равен ключу в словаре)</param>
    public Player(WebSocket socket, string playerID)
    {

        Socket = socket;
        ID = playerID;
        var data = new JObject();
        data["text"] = "0"; // You added to the server.
        data["playerID"] = ID;
        SendData(
            new Сommunication.Responce(
                Сommunication.MessageType.Status, "PlayerStatus", data));
    }

}