﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

/// <summary>
/// Представляет запрос/ответ между клиентом и сервером
/// </summary>
public class Сommunication
{
    /// <summary>
    /// Запрос от игрока
    /// </summary>
    public class Request
    {
        /// <summary>
        /// Параметры запроса
        /// </summary>
        JObject variables;

        /// <summary>
        /// Сырой формат запроса
        /// </summary>
        public string Source;

        /// <summary>
        /// Запрос от игрока
        /// </summary>
        /// <param name="data">Данные в JSON</param>
        public Request(string data)
        {
            try
            {
                Source = data;
                // Попытаемся десериализовать JSON данные...
                variables = JsonConvert.DeserializeObject<JObject>(data);
            }
            catch
            {
                // В случае ошибки десериализации данных обозначим список параметров как null.
                variables = null;
            }
        }

        /// <summary>
        /// Дает доступ к коллекции параметров
        /// </summary>
        /// <param name="name">Имя параметра</param>
        public JToken this[string name]
        {
            get
            {
                try
                {
                    // Попытаемся вернуть искомый параметр...
                    return variables[name];
                }
                catch
                {
                    // В случае ошибки вернет параметр как null.
                    return null;
                }
            }
        }
    }

    public enum MessageType { Status, Command, Warning, Error, ServerError }

    public class Responce
    {
        /// <summary>
        /// Тип ответа
        /// </summary>
        [JsonProperty("type")]
        public string Type;

        /// <summary>
        /// Название события
        /// </summary>
        [JsonProperty("eventName")]
        public string EventName;

        /// <summary>
        /// Данные
        /// </summary>
        [JsonProperty("data")]
        JObject Data;

        //[JsonIgnore]

        public Responce(MessageType type, string eventName, JObject data)
        {

            Type = type.ToString();
            EventName = eventName;
            // data.Add("RoomsCount",Global.Rooms.Count.ToString());
            Data = data;
        }
    }
}
