// <Настройки приложения>
var Settings = {
    cellSize: 24, // Размер клетки
    gridOn: true, // Отображение сетки
    gridColor: "#c6c6c6", // Цвет сетки
    gridDash: false, // Отображение пунктира сетки
    gridDashA: 5, // Длина пунктира
    gridDashB: 3, // Длина между пунктирами
    cellUnknowColor: "#fff", // Фон неизвестной клетки
    cellKnowColor: "#f6f6f6", // Фон известной клетки
    footages: {},
    nickname: "Opponent",
    lastB: null,
    last_mine: false,
    timer: null,
    timeout_sec: 25,
    sound_on: true,
    sound_volume: 0.3,
    isHelp: false,
    opponentNick: null,
};

var statusCodes = {};
statusCodes['100'] = 'You are already in the room.';
statusCodes['250'] = 'Opponent has left the game.';
statusCodes['400'] = 'The room has been removed.';
statusCodes['450'] = 'Socket has been closed.';
statusCodes['0'] = 'You added to the server.';

// Инициализаторы
var renderField = null,
    ctx = null,
    playerID = null,
    roomID = null,
    turnBy = null,
    __MINES_COUNT = 0;


// Подбирает оптимальное значение разрешения изображения
function getResolution(r) {
    return Math.min(256, Math.max(32, Math.pow(2, Math.ceil(Math.log(r) / Math.log(2)))));
}

function loadFootages() {
    var res = getResolution(Settings.cellSize);
    Settings.footages['mineFound'] = new Image();
    Settings.footages['mineFound'].src = 'images/swiper/hd/flag_' + res + '.png?v1.101';

    Settings.footages['mineUnknow'] = new Image();
    Settings.footages['mineUnknow'].src = 'images/swiper/hd/unknow_' + res + '.png?v1.101';

    Settings.footages['lastB'] = new Image();
    Settings.footages['lastB'].src = 'images/swiper/hd/opponent_last_step_' + res + '.png?v1.101';

    Settings.footages['mine'] = new Image();
    Settings.footages['mine'].src = 'images/swiper/hd/mine_' + res + '.png?v1.101';

    var footages_loadCounter = 0;
    for (var footage in Settings.footages) {
        Settings.footages[footage].onload = function() {
            if (++footages_loadCounter == Object.keys(Settings.footages).length) {
                now.createField();
            }
        };
    }
}

// Игровое поле
var Field = function() {
    // клетки поля
    this.cells = [];
    this.openCells = [];
    this.flagCells = new Set();
    this.minesProcent = function(proc) {
        this.updateMinesCount(proc);
        return proc;
    };
    this.minesCount = 0;
    // обновляет кол-во мин на основе процентов
    this.updateMinesCount = function(minesProcent) {
        this.minesCount = Math.floor((this.size.x * this.size.y) * (minesProcent / 100) + 0.5);
    };
    // возвращает кол-во клеток в поле
    this.getCellsCount = function() {
        return this.size.x * this.size.y;
    };
    // получает кол-во оставшихся мин (префискное условие!)
    this.getCellsCountToFrom = function(x, y) {
        return this.getCellsCount() - x * this.size.y - y;
    };
    // Размер поля
    // x: координата X
    // y: координата Y
    var Size = function(x, y) {
        // Задает новый размер
        this.Resize = function(x, y) {
            this.x = x;
            this.y = y;
            renderField.width = x * Settings.cellSize + 2;
            renderField.height = y * Settings.cellSize + 2;
        }
    };
    this.size = new Size();
    // Объект координат клетки
    // x: координата X
    // y: координата Y
    var Position = function(x, y) {
        this.x = x; // координата X
        this.y = y; // координата Y
        // Задает новую позицию
        // x: координата X
        // y: координата Y
        this.newPosition = function(x, y) {
            this.x = x;
            this.y = y;
        }
    };
    // Объект клетки поля
    var Cell = function() {
        //позиция клетки
        this.position = null;
        //заминирована ли клетка
        this.flag = false;
        // показана ли клетка
        this.showed = false;
        // сколько мин вокруг
        this.around = 0;
    };
    // Возвращает кол-во мин вокруг клетки
    this.getMinesAround = function(x, y) {
        var count = 0;
        for (var i = -1; i < 2; ++i)
            for (var j = -1; j < 2; ++j) {
                if (i == 0 && j == 0) continue;
                var posX = x + i,
                    posY = y + j;
                if ((posX >= 0 && posX < this.size.x) && (posY >= 0 && posY < this.size.y))
                    if (this.cells[posX][posY].flag)
                        ++count;
            }
        return count;
    }
    this.isMatchInFlags = function(x, y) {
        var found = false;
        var _ttX1 = Array.from(this.flagCells);
        for (var i = 0; i < _ttX1.length; ++i) {
            if (_ttX1[i][0] == x && _ttX1[i][1] == y) {
                found = true;
                break;
            }
        }
        return found;
    }
    this.removeInFlags = function(x, y) {
        if (x == -1 && y == -1) {
            this.flagCells = new Set();
        } else {
            var _ttX1 = Array.from(this.flagCells);
            for (var i = 0; i < _ttX1.length; ++i) {
                if (_ttX1[i][0] == x && _ttX1[i][1] == y) {
                    this.flagCells.delete(_ttX1[i]);
                }
            }
        }
        $("#flagsCount_Label").html(this.flagCells.size);
        $("#minesMCount_Label").html(__MINES_COUNT - this.flagCells.size);
    }


    // Помечает клетку как хинт от оппонента
    this.setHint = function(x, y) {
        if (!now.cells[x][y].showed) {
            ctx.fillStyle = "rgba(0,255,36,0.2)";
            ctx.fillRect(Settings.cellSize * x, Settings.cellSize * y, ~~(Settings.cellSize) + 0.5, ~~(Settings.cellSize) + 0.5);
        }
    }

    // Отправляет оппоненту ваш хинт
    this.sendHint = function(x, y) {
        if (!now.cells[x][y].showed) {
            socket.send($.toJSON({
                "method": "sendHint",
                "x": x,
                "y": y
            }));
        }
    }




    this.setFlag = function(x, y) {
        if (x == -1 && y == -1) {
            this.removeInFlags(x, y);
        } else {
            if (this.cells[x][y].showed == false) {
                if (this.isMatchInFlags(x, y)) this.removeInFlags(x, y);
                else this.flagCells.add([x, y]);
                this.createField();
            }
            if (this.flagCells.size == __MINES_COUNT) {
                var a = $.toJSON({
                    "method": "testFlags",
                    "flags": Array.from(this.flagCells)
                });
                socket.send(a);
            }
        }
        $("#flagsCount_Label").html(this.flagCells.size);
        $("#minesMCount_Label").html(__MINES_COUNT - this.flagCells.size);
    }

    this.setColorSolidOnCanvas = function(win) {
        ctx.beginPath();
        ctx.rect(0, 0, renderField.width, renderField.height);
        ctx.closePath();

        switch (win) {
            case 0:
                // Установка черного солида на канвас
                ctx.fillStyle = "rgba(115,115,115,0.25)";
                break;
            case 1:
                // Установка зеленого солида на канвас
                ctx.fillStyle = "rgba(97,230,0,0.25)";
                break;
            case 2:
                // Установка серого солида на канвас
                ctx.fillStyle = "rgba(192,192,192,0.25)";
                break;
            default:
                // Установка серого солида на канвас
                ctx.fillStyle = "rgba(192,192,192,0.25)";
                break;
        }
        ctx.fill();
    }



    //инициализация поля
    this.createNullField = function() {
        for (var i = 0; i < this.size.x; ++i) {
            this.cells[i] = [];
            for (var j = 0; j < this.size.y; ++j) {
                this.cells[i][j] = new Cell();
                this.cells[i][j].position = new Position(i, j);
                this.cells[i][j].around = 0;
            }
        }
    };
    this.createGrid = function() {
        for (var i = 0; i <= this.size.y; ++i) {
            ctx.beginPath();
            ctx.lineWidth = 1;
            ctx.strokeStyle = Settings.gridColor;
            if (Settings.gridDash) ctx.setLineDash([Settings.gridDashA, Settings.gridDashB]);
            ctx.moveTo(0, ~~(Settings.cellSize * i) + 0.5);
            ctx.lineTo(~~(Settings.cellSize * this.size.x) + 0.5, ~~(Settings.cellSize * i) + 0.5);
            ctx.stroke();
        }
        for (var i = 0; i <= this.size.x; ++i) {
            ctx.beginPath();
            ctx.lineWidth = 1;
            ctx.strokeStyle = Settings.gridColor;
            if (Settings.gridDash) ctx.setLineDash([Settings.gridDashA, Settings.gridDashB]);
            ctx.moveTo(~~(Settings.cellSize * i) + 0.5, 0);
            ctx.lineTo(~~(Settings.cellSize * i) + 0.5, ~~(Settings.cellSize * this.size.y) + 0.5);
            ctx.stroke();
        }
    };
    this.getCellFontColor = function(minesAround) {
            switch (minesAround) {
                case 1:
                    return "#0000ff";
                case 2:
                    return "#008000";
                case 3:
                    return "#cc0000";
                case 4:
                    return "#004c8a";
                case 5:
                    return "#960018";
                case 6:
                    return "#bb00a7";
                case 7:
                    return "#000";
                case 8:
                    return "#ff0000";
                default:
                    return "#000";
            }
        }
        // Отрисовка поля
    this.showField = function() {
        if (Settings.gridOn) {
            this.createGrid();
        }
        for (var i = 0; i < this.size.x; ++i) {
            for (var j = 0; j < this.size.y; ++j) {
                ctx.fillStyle = Settings.cellKnowColor;
                ctx.fillRect(Settings.cellSize * i + 1, Settings.cellSize * j + 1, Settings.cellSize - 1, Settings.cellSize - 1);
                if (this.cells[i][j].showed) {
                    if (this.isMatchInFlags(i, j)) {
                        this.removeInFlags(i, j);
                        $("#flagsCount_Label").html(this.flagCells.size);
                        $("#minesMCount_Label").html(__MINES_COUNT - this.flagCells.size);
                    }
                    if (this.cells[i][j].around != 0) {
                        // показать кол-во мин в клетке
                        ctx.textAlign = "center";
                        ctx.textBaseline = "middle";
                        ctx.fillStyle = this.getCellFontColor(this.cells[i][j].around);
                        ctx.font = "bold " + ~~(Settings.cellSize / 1.8) + "pt Arial";
                        ctx.fillText(this.cells[i][j].around, ~~(Settings.cellSize * i + Math.floor(Settings.cellSize / 2)) + 0.5, ~~(Settings.cellSize * j + Math.floor(Settings.cellSize / 2 + 2)) + 0.5);
                    }
                } else {
                    if (this.cells[i][j].flag) {
                        // Если флаг в клетке
                        ctx.drawImage(Settings.footages['mineUnknow'], Settings.cellSize * i, Settings.cellSize * j, ~~(Settings.cellSize) + 0.5, ~~(Settings.cellSize) + 0.5);
                        ctx.drawImage(Settings.footages['mineFound'], Settings.cellSize * i + 2, Settings.cellSize * j + 2, Settings.cellSize - 5, Settings.cellSize - 5);
                    } else {
                        ctx.drawImage(Settings.footages['mineUnknow'], Settings.cellSize * i, Settings.cellSize * j, ~~(Settings.cellSize) + 0.5, ~~(Settings.cellSize) + 0.5);
                    }
                }
            }
        }
        for (var i = 0; i < this.size.x; ++i) {
            for (var j = 0; j < this.size.y; ++j) {
                if (Settings.last_mine) {
                    if (!this.cells[i][j].showed) {
                        ctx.drawImage(Settings.footages['mine'], Settings.cellSize * i, Settings.cellSize * j, ~~(Settings.cellSize) + 0.5, ~~(Settings.cellSize) + 0.5);
                    }
                }
                if (Settings.lastB != null) {
                    if (Settings.lastB[0] == i && Settings.lastB[1] == j) {
                        ctx.drawImage(Settings.footages['lastB'], Settings.cellSize * i - 7, Settings.cellSize * j - 7, Settings.cellSize + 15, Settings.cellSize + 15);
                    }
                }
            }
        }
    };
    this.clearField = function() {
            ctx.clearRect(0, 0, renderField.width, renderField.height);
        }
        // Устанавливает парметры поля, которые известны
    this.setCells = function() {
            for (var i = 0; i < this.openCells.length; ++i) {
                this.cells[this.openCells[i][0]][this.openCells[i][1]].showed = true;
                this.cells[this.openCells[i][0]][this.openCells[i][1]].around = this.openCells[i][2];
            }
            var _ttX1 = Array.from(this.flagCells);
            for (var i = 0; i < _ttX1.length; ++i) {
                this.cells[_ttX1[i][0]][_ttX1[i][1]].flag = true;
            }
        }
        // Создает поле
    this.createField = function() {
        this.clearField();
        this.createNullField();
        this.setCells();
        this.showField();
    }
};

var now = new Field(),
    socket;

// Массив непомеченных клеток вокруг клетки
function nonFlagsArray(x, y) {
    var a = new Set();
    var count = 0;
    for (var i = -1; i < 2; ++i)
        for (var j = -1; j < 2; ++j) {
            if (i == 0 && j == 0) continue;
            var posX = x + i,
                posY = y + j;
            if ((posX >= 0 && posX < now.size.x) && (posY >= 0 && posY < now.size.y)) {
                if (!now.cells[posX][posY].flag && !now.cells[posX][posY].showed) {
                    a.add([posX, posY]);
                }
            }
        }
    return Array.from(a);
}
// Кол-во флагов вокруг клетки
function flagsAround(x, y) {
    var count = 0;
    for (var i = -1; i < 2; ++i)
        for (var j = -1; j < 2; ++j) {
            if (i == 0 && j == 0) continue;
            var posX = x + i,
                posY = y + j;
            if ((posX >= 0 && posX < now.size.x) && (posY >= 0 && posY < now.size.y)) {
                if (now.cells[posX][posY].flag) {
                    if (now.cells[posX][posY].showed) {
                        now.removeInFlags(x, y);
                    } else {
                        ++count;
                    }
                }
            }
        }
    return count;
}

function openCell(x, y) {
    if (turnBy == playerID) {
        // Проверим, является ли клетка открытой
        if (now.cells[x][y].showed) {
            var arF = flagsAround(x, y); // Флагов вокруг
            var arM = now.cells[x][y].around // Мин вокруг
                // Если кол-во флагов совпадает с кол-вом мин => взорвем все остальное
            if (arF == arM && arM != 0) {
                var nonflags = nonFlagsArray(x, y);
                if (nonflags.length > 0) {
                    socket.send($.toJSON({
                        "method": "openCell",
                        "cells": nonflags,
                        "byX": x,
                        "byY": y
                    }));
                    sound_Your_Best_Turn();
                }
            }
        } else {
            var f = false;
            f = now.cells[x][y].flag;
            if (!f) {
                var a = new Set();
                a.add([x, y]);
                socket.send($.toJSON({
                    "method": "openCell",
                    "cells": Array.from(a),
                    "byX": x,
                    "byY": y
                }));
                sound_Your_Turn()
            }
        }
    }
}


function resetHelp() {
    Settings.isHelp = false;
    document.getElementById('opponentNick_Label').style.background = '#ff3a23';
    document.getElementById('opponentNick_Label').style.color = '#fff';
    $("#opponentNick_Label").html(Settings.opponentNick);
}


function mouseClick(evt) {
    if (roomID != null) {
        var mouseX = evt.pageX - renderField.offsetLeft;
        var mouseY = evt.pageY - renderField.offsetTop;
        var x = ~~(mouseX / Settings.cellSize);
        var y = ~~(mouseY / Settings.cellSize);

        if (evt.which == 1) {
            // Левый клик мыши
            if (Settings.isHelp) {
                now.sendHint(x, y);
            } else {
                openCell(x, y);
            }
        }
        if (evt.which == 2) {
            // Средний клик мыши
            now.sendHint(x, y);
        }
        if (evt.which == 3) {
            // Правый клик мыши
            now.setFlag(x, y);
        }
    }
}

function roomFind() {
    document.getElementById('roomFind_Button').disabled = true;
    document.getElementById('roomFind_Button').value = "...";
    if (roomID != null) {
        roomID = null
    }
    if (playerID == null) {
        connectToServer();
        $("#searchStatus").html("Connecting to the server... ");
    } else {
        socket.send($.toJSON({
            "method": "roomFind",
            "nickname": Settings.nickname
        }));
        $("#searchStatus").html("Searching for rooms...");
    }
}

function startTimer() {
    clearTimeout(Settings.timer);
    var time = $("#timer_label").html();
    if (time == 0) {
        return;
    }
    if (time == 11 && turnBy == playerID) {
        sound_TickTock(true);
    }
    $("#timer_label").html(--time);
    Settings.timer = setTimeout(startTimer, 1000);
}


function connectToServer() {
    if (typeof(WebSocket) !== 'undefined') {
        socket = new WebSocket("wss://rmsweepersrv.azurewebsites.net/SocketHandler.ashx");
    } else {
        socket = new MozWebSocket("wss://rmsweepersrv.azurewebsites.net/SocketHandler.ashx");
    }
    socket.onopen = function(msg) {
        socket.send($.toJSON({
            "method": "getManualID"
        }));
    }
    socket.onmessage = function(msg) {
        var answer = $.parseJSON(msg.data);
        if (answer.eventName == "RoomInformation") {
            now.openCells = answer.data.openCells;
            Settings.lastB = answer.data.lastB;
            now.createField();
            if (answer.data.itEnd) {
                // Подрыв на мине
                Settings.last_mine = true;
                Settings.lastB = answer.data.explodedAt;
                now.createField();
            }
            if (roomID == null) {
                // Новая игра инициализировалась.
                now.setFlag(-1, -1); // Уберет все флажки с поля.   
                Settings.last_mine = false;
                now.createField();
                $("#chat_messages").html("<span style='padding: 3px; color:#808080'>Feel free to talk!</span>");
                roomID = answer.data.roomID;
                __MINES_COUNT = answer.data.minesCount;
                $("#minesCount_Label").html(__MINES_COUNT);
                $("#minesMCount_Label").html("0");
                $("#flagsCount_Label").html("0");
                $("#searchStatus").html("•••");
                $("#left_Label").html("<span style='font-size: 20px;'><span style='font-family: RobotoRegular'><span id='timer_label'>N</span></span> sec left...</span>");
                startTimer();
                Settings.opponentNick = answer.data.opponentNick;
                $("#opponentNick_Label").html(Settings.opponentNick);
                $("#logkek").html("");
                document.getElementById('roomFind_Button').value = "The game has begun!";
                sound_NewGame();
            }
            var dd = GetUnixTime() - answer.data.timestamp;
            dd = dd > 0 ? dd : 0;
            $("#timer_label").html((Settings.timeout_sec + 1) - dd);
            startTimer();
            turnBy = answer.data.turnBy;
            resetHelp();
            if (turnBy == playerID) {
                sound_Opponent_Turn();
            } else {
                sound_TickTock(false);
            }
            $("#timeouts_S_Label").html(answer.data.timeoutsS);
            $("#timeouts_A_Label").html(answer.data.timeoutsA);
            refreshWhoColor(turnBy == playerID, "");
        }
        if (answer.eventName == "PlayerStatus") {
            if (typeof(answer.data.cause) !== 'undefined') {
                // Конец катки    
                resetHelp();
                roomID = null;
                now.createField();
                $("#flagsCount_Label").html(0);
                $("#minesMCount_Label").html(0);
                $("#timeouts_S_Label").html(0);
                $("#timeouts_A_Label").html(0);
                $("#searchStatus").html("•••");
                document.getElementById('roomFind_Button').disabled = false;
                document.getElementById('roomFind_Button').value = "Start the game";
                if (answer.data.text == "You win!") {
                    now.setColorSolidOnCanvas(1);
                    refreshWhoColor(4, answer.data.cause);
                } else {
                    now.setColorSolidOnCanvas(0);
                    refreshWhoColor(3, answer.data.cause);
                }
                socket.close();
            }
            if (typeof(answer.data.playerID) !== 'undefined') {
                if (playerID == null) {
                    playerID = answer.data.playerID;
                    roomFind();
                }
            }
        }
        if (answer.data.text == 100) {
            $("#searchStatus").html(answer.data.text);
        }
        if (answer.eventName == "RoomStatusUpdate") {
            if (roomID != null) {
                if (answer.data.text == 400) {
                    roomID = null;
                    $("#searchStatus").html("•••");
                    document.getElementById('roomFind_Button').disabled = false;
                    document.getElementById('roomFind_Button').value = "Start the game";
                    now.setColorSolidOnCanvas(1);
                    refreshWhoColor(4, statusCodes['250']);
                }
                socket.close();
            }
        }
        if (answer.eventName == "NewMsg") {
            $('#chat_messages').append('<div id="mgs_st_o"><div>' + answer.data.msg + '</div></div>').show();
            document.getElementById("chat_messages").scrollTop = document.getElementById("chat_messages").scrollHeight;
        }
        if (answer.eventName == "NewHint") {
            now.setHint(answer.data.x, answer.data.y);
        }


        // Если возникла ошибка сервера - отобразить в дебаг-лист.
        if (answer.type == "ServerError") {
            var $el = document.createElement('p');
            $el.innerHTML = ':> ' + msg.data;
            document.getElementById('logkek').appendChild($el);
        }
    };
    socket.onerror = function(e) {
        resetHelp();
        var $el = document.createElement('p');
        $el.innerHTML = ':> ' + e.message;
        document.getElementById('logkek').appendChild($el);
    }
    socket.onclose = function(event) {
        Settings.opponentNick = "Opponent";
        resetHelp();
        if (~$("#turnBy_Label").html().indexOf("turn")) {
            refreshWhoColor(2, statusCodes['450']);
        }
        clearTimeout(Settings.timer);
        last_mine = false;
        roomID = null;
        playerID = null;
        $("#searchStatus").html("•••");
        document.getElementById('roomFind_Button').disabled = false;
        document.getElementById('roomFind_Button').value = "Start the game";
    };
}

function GetUnixTime() {
    return Math.floor(Date.now() / 1000);
}


function msgToChat() {
    if (roomID != null) {
        var nn = $("#msgToChat_textbox").val();
        if (nn.length == 0 || nn.length > 255) {
            alert("Incorrect length! Max: 20, Your: " + nn.length);
            return;
        }
        $('#chat_messages').append('<div id="mgs_st_y"><div>' + $('<div/>').text(nn).html() + '</div></div>').show();
        socket.send($.toJSON({
            "method": "toChat",
            "msg": $("#msgToChat_textbox").val()
        }));
        $("#msgToChat_textbox").val("");
        document.getElementById("chat_messages").scrollTop = document.getElementById("chat_messages").scrollHeight;
    }
}


function friendlyLobby_Create() {
    document.getElementById('roomFind_Button').disabled = true;
    document.getElementById('roomFind_Button').value = "...";
    if (roomID != null) {
        roomID = null
    }
    if (playerID == null) {
        connectToServer();
        $("#searchStatus").html("Connecting to the server... ");
    } else {
        socket.send($.toJSON({
            "method": "createLobby",
            "nickname": Settings.nickname
        }));
        $("#searchStatus").html("Waiting for friend...");
    }

}

function toSurrender() {
    if (roomID != null) {
        if (confirm('Are you sure?')) {
            socket.close();
            now.setColorSolidOnCanvas(2);
            refreshWhoColor(2, "");
        }
    } else {
        // friendlyLobby_Create();
    }
}



function helpStart() {
    if (roomID != null && turnBy != playerID) {
        if (Settings.isHelp) {
            resetHelp();
        } else {
            Settings.isHelp = true;
            document.getElementById('opponentNick_Label').style.background = '#000';
            document.getElementById('opponentNick_Label').style.color = '#fff';
            document.getElementById('opponentNick_Label').innerHTML = 'HELP MODE: ON';
        }
    }
}


function refreshWhoColor(status, text) {
    if (status == 1) {
        document.getElementById('parallelogram').style.background = '#23aaff';
        document.getElementById('tablo').style.background = '#8ad2ff';
        $("#turnBy_Label").html("your turn");
    }
    if (status == 0) {
        document.getElementById('parallelogram').style.background = '#e32323';
        document.getElementById('tablo').style.background = '#ed7b7b';
        $("#turnBy_Label").html("opponent's turn");
    }
    if (status == 2) {
        document.getElementById('parallelogram').style.background = '#d4d4d4';
        document.getElementById('tablo').style.background = '#e0e0e0';
        $("#turnBy_Label").html("the game is inactive");
    }
    if (status == 3) {
        document.getElementById('parallelogram').style.background = '#1a1a1a';
        document.getElementById('tablo').style.background = '#333333';
        $("#turnBy_Label").html("You Lose!");
        sound_Lose();
    }
    if (status == 4) {
        document.getElementById('parallelogram').style.background = '#00e600';
        document.getElementById('tablo').style.background = '#32cd32';
        $("#turnBy_Label").html("You Win!");
        sound_Win();
    }
    if (text != "") {
        $("#left_Label").html(text);
    }
}

function RulesPopUpShow(show) {
    if (show) {
        document.getElementById('rulesOpen_Label').onclick = function() { RulesPopUpShow(false); };
        AboutPopUpShow(false);
        SettingsPopUpShow(false);
        $("#rules_popup").slideDown(300);
    } else {
        document.getElementById('rulesOpen_Label').onclick = function() { RulesPopUpShow(true); };
        $("#rules_popup").slideUp(300);
    }
}

function SettingsPopUpShow(show) {

    if (show) {
        document.getElementById('youNick_Label').onclick = function() { SettingsPopUpShow(false); };
        document.getElementById('settingsOpen_Label').onclick = function() { SettingsPopUpShow(false); };
        AboutPopUpShow(false);
        RulesPopUpShow(false);
        $("#settings_popup").slideDown(300);
    } else {
        document.getElementById('youNick_Label').onclick = function() { SettingsPopUpShow(true); };
        document.getElementById('settingsOpen_Label').onclick = function() { SettingsPopUpShow(true); };
        $("#settings_popup").slideUp(300);
    }
}

function AboutPopUpShow(show) {


    if (show) {
        document.getElementById('whatisit').onclick = function() { AboutPopUpShow(false); };
        SettingsPopUpShow(false);
        RulesPopUpShow(false);
        $("#about_popup").slideDown(300);
    } else {
        document.getElementById('whatisit').onclick = function() { AboutPopUpShow(true); };
        $("#about_popup").slideUp(300);
    }
}


// Корректносить ника
function isCorrectName(str) {
    return /^[a-zA-Zа-яА-Я0-9: \.'\-_!@\^\?]{1,}$/.test(str);
}

function newNickname() {
    var nn = $("#nickname").val();
    if (isCorrectName(nn)) {
        if (nn.length > 15 || nn.length == 0) {
            alert("Incorrect length! Max: 15, Your: " + nn.length);
            return;
        }
        Settings.nickname = nn;
        $("#youNick_Label").text(Settings.nickname);
    } else {
        alert("Incorrect characters!");
        return;
    }
}
// <Раздел музыкальных функций>
var audio_Win = new Audio('sounds/Win.wav');

function sound_Win() {
    if (Settings.sound_on) {
        audio_Win.volume = Settings.sound_volume;
        audio_Win.play();
    }
}
var audio_Lose = new Audio('sounds/Lose.wav');

function sound_Lose() {
    if (Settings.sound_on) {
        audio_Lose.volume = Settings.sound_volume;
        audio_Lose.play();
    }
}
var audio_Opponent_Turn = new Audio('sounds/Opponent_Turn.wav');

function sound_Opponent_Turn() {
    if (Settings.sound_on) {
        audio_Opponent_Turn.volume = Settings.sound_volume;
        audio_Opponent_Turn.play();
    }
}
var audio_Your_Turn = new Audio('sounds/Your_Turn.wav');

function sound_Your_Turn() {
    if (Settings.sound_on) {
        audio_Your_Turn.volume = Settings.sound_volume;
        audio_Your_Turn.play();
    }
}
var audio_Your_Best_Turn = new Audio('sounds/Your_Turn.wav');

function sound_Your_Best_Turn() {
    if (Settings.sound_on) {
        audio_Your_Best_Turn.volume = Settings.sound_volume;
        audio_Your_Best_Turn.play();
    }
}
var audio_NewGame = new Audio('sounds/New_Game.wav');

function sound_NewGame() {
    if (Settings.sound_on) {
        audio_NewGame.volume = Settings.sound_volume;
        audio_NewGame.play();
    }
}
var audioTickTock = new Audio('sounds/Tick_Tock.mp3');

function sound_TickTock(show) {
    if (show) {
        if (Settings.sound_on) {
            audioTickTock.volume = Settings.sound_volume;
            audioTickTock.play();
        }
    } else {
        audioTickTock.pause();
        audioTickTock.currentTime = 0;
    }
}

// </Раздел музыкальных функций>


function soundTurn() {
    Settings.sound_on = Settings.sound_on ? false : true;
    $('#turnSound_Label').html((Settings.sound_on ? "disable" : "enable") + " sounds");
}

$(document).ready(function() {
    refreshWhoColor(2, "");
    renderField = document.getElementById("renderField");
    renderField.onmousedown = mouseClick;

    ctx = renderField.getContext('2d');
    now.size.Resize(30, 20);
    loadFootages();
    $("canvas").on("contextmenu", false);
});

$(document).click(function(e) {
    if ($(e.target).closest(".itpopup").length == 0 && $(e.target).closest(".popup_opener").length == 0) {
        AboutPopUpShow(false);
        SettingsPopUpShow(false);
        RulesPopUpShow(false);
    }
});
